#!/usr/bin/env python
#
# Script written by Sara Bosticardo and Simona Schiavi
# If you need further assistance, to report any bug or for suggestions
# please contact: sara.bosticardo@univr.it or simona.schiavi@polytechnique.edu 
# 
#----------------------------------------------------------------------------------------------------------------------------------------------
#
# This script is created to run in a folder organized as follow:
# Main_Analysis_folder/PZ_number/Diffusion
# where "_number" should be the relative number assigned to each subject and the folder "Diffusion" of each subject contains:
# - AP folder with inside the "merged" diffusion images acquired in AP (data_AP.nii) and the corresponding data_AP.bvecs data_AP.bvals
# - PA folder with inside the "merged" b=0 images acquired in PA (data_PA.nii) and the corresponding data_PA.bvecs data_PA.bvals
#
# to create the "acqparams.txt" file at line 106 I assumed a total of 24 b=0 images (12 acquired as AP and 12 as PA) and a readout time of 0.032
#
# !!!!!!!!!!!!!!!! Please at line 125 change the "149" with the total number of volumes in your acquisitions AP+PA !!!!!!!!!!!!!!!!!!!!!!!!!!!!!
#
#----------------------------------------------------------------------------------------------------------------------------------------------
#
import numpy as np
import os
import shutil

spath = os.path.join('/...', '...', '...', '...', '...') # main analysis folder

Subj_list = [f for f in os.listdir(spath) if not f.startswith('.')]
Subj_list.sort()

if len(Subj_list) == 0:
    print(f"Warning! Files not found in {spath}. Please provide the data in the folder.")

for subj in Subj_list:
    print(f"======================= SUBJ-{subj} =======================")

    subj_folder = os.path.join(spath, subj, 'Diffusion')
    #
    #-----------------------------------------------------------------------------------------------------------------
    # Step 1: denoise the image through dwidenoise of MRtrix
    #
    AP_folder = os.path.join(subj_folder, 'AP')
    AP_raw = os.path.join(AP_folder, 'data_AP.nii')
    AP_denoise = os.path.join(AP_folder, 'data_AP_denoised.nii.gz')
    noise_map = os.path.join(AP_folder, 'noise_map.nii.gz')
    cmd = f'dwidenoise {AP_raw} {AP_denoise} -noise {noise_map}'
    os.system(cmd)

    PA_folder = os.path.join(subj_folder, 'PA')
    PA_raw = os.path.join(PA_folder, 'data_PA.nii')
    PA_denoise = os.path.join(PA_folder, 'data_PA_denoised.nii.gz')
    noise_map = os.path.join(PA_folder, 'noise_map.nii.gz')
    cmd = f'dwidenoise {PA_raw} {PA_denoise} -noise {noise_map}'
    os.system(cmd)
    #
    #-----------------------------------------------------------------------------------------------------------------
    # Step 2: bet of only AP data
    AP_denoise_bet = os.path.join(AP_folder, 'data_AP_denoised_bet')
    cmd = f'bet {AP_denoise} {AP_denoise_bet} -f 0.2 -m -R -v'
    os.system(cmd)
    #
    #-----------------------------------------------------------------------------------------------------------------
    # Step 3: extracting the b=0 images from AP acquisition
    b0_AP = os.path.join(AP_folder, 'b0_AP.nii.gz')
    AP_bvecs = os.path.join(AP_folder, 'data_AP.bvecs')
    AP_bvals = os.path.join(AP_folder, 'data_AP.bvals')
    cmd = f'dwiextract -bzero {AP_denoise} {b0_AP} -fslgrad {AP_bvecs} {AP_bvals}'
    #
    #-----------------------------------------------------------------------------------------------------------------
    # Step 4: topup and eddy correction
    b0_PA = os.path.join(PA_folder, 'b0_PA.nii.gz')
    shutil.copyfile(PA_denoise, b0_PA)
    eddy_topup_folder = os.path.join(subj_folder, 'eddy_topup')
    os.makedirs(eddy_topup_folder, exist_ok=True)
    b0_AP_PA = os.path.join(eddy_topup_folder, 'b0_AP_PA.nii.gz')

    cmd = f'fslmerge -t {b0_AP_PA} {b0_AP} {b0_PA}'
    os.system(cmd)
    os.chdir(eddy_topup_folder)

    acqparams = os.path.join(eddy_topup_folder, 'acqparams.txt')
    if os.path.exists(acqparams):
        os.remove(acqparams)

    f = open(acqparams, 'w+')
    f.write("0 -1 0 0.0958793\n0 -1 0 0.0958793\n0 -1 0 0.0958793\n0 -1 0 0.0958793\n0 -1 0 0.0958793\n0 -1 0 0.0958793\n0 -1 0 0.0958793\n0 -1 0 0.0958793\n0 -1 0 0.0958793\n0 -1 0 0.0958793\n0 -1 0 0.0958793\n0 -1 0 0.0958793\n0 1 0 0.0958793\n0 1 0 0.0958793\n0 1 0 0.0958793\n0 1 0 0.0958793\n0 1 0 0.0958793\n0 1 0 0.0958793\n0 1 0 0.0958793\n0 1 0 0.0958793\n0 1 0 0.0958793\n0 1 0 0.0958793\n0 1 0 0.0958793\n0 1 0 0.0958793\n")
    f.close()

    topup_out = os.path.join(eddy_topup_folder, "my_topup_results")
    topup_iout = os.path.join(eddy_topup_folder, "my_hifi_b0")
    cmd = f'topup --imain={b0_AP_PA} --datain={acqparams} --config=b02b0.cnf --out={topup_out} --iout={topup_iout} --verbose'
    os.system(cmd)
    #
    cmd = f'fslmaths {topup_iout} -Tmean {topup_iout}'
    os.system(cmd)
    topup_bet = os.path.join(eddy_topup_folder, "my_hifi_b0_brain")
    cmd = f'bet {topup_iout} {topup_bet} -m -v'
    os.system(cmd)
    topup_bet_mask = os.path.join(eddy_topup_folder, "my_hifi_b0_brain_mask.nii.gz")
    #
    eddy_index = os.path.join(eddy_topup_folder, "index.txt")
    feddy = open(eddy_index, 'w+')
    for i in range(149):
        feddy.write("1 ")
    feddy.close()
    #
    eddy_result = os.path.join(eddy_topup_folder, "dwi_preproc")
    cmd = f"eddy --imain={AP_denoise} --mask={topup_bet_mask} --acqp={acqparams} --index={eddy_index} --bvecs={AP_bvecs} --bvals={AP_bvals} --topup={topup_out} --out={eddy_result} --repol --data_is_shelled --verbose"   
    os.system(cmd)
    #
    #-----------------------------------------------------------------------------------------------------------------
    # Step 5: Copying the results in the folder "Diffusion"
    dwi_preproc_eddy = os.path.join(eddy_topup_folder, 'dwi_preproc.nii.gz')
    dwi_preproc = os.path.join(subj_folder, 'dwi_preproc.nii.gz')
    shutil.copyfile(dwi_preproc_eddy, dwi_preproc)
    dwi_preproc_eddy_bvecs = os.path.join(eddy_topup_folder, 'dwi_preproc.eddy_rotated_bvecs')
    bvecs = os.path.join(subj_folder, 'dwi_preproc.bvecs')
    shutil.copyfile(dwi_preproc_eddy_bvecs, bvecs)
    bvals = os.path.join(subj_folder, 'dwi_preproc.bvals')
    shutil.copyfile(AP_bvals, bvals)
    #
    #-----------------------------------------------------------------------------------------------------------------
    # Step 6: Performing B1 bias correction with dwibiascorrect of MRtrix and ANTS
    #
    dwi_preproc_b1corr = os.path.join(subj_folder, 'dwi_preproc_B1corr.nii.gz')
    bias_b1 = os.path.join(subj_folder, 'bias_B1.nii.gz')
    cmd = f'dwibiascorrect {dwi_preproc} {dwi_preproc_b1corr} -ants -bias {bias_b1} -fslgrad {bvecs} {bvals}'
    os.system(cmd)
 
