
#!/usr/bin/env python
#
# Script written by Sara Bosticardo
# If you need further assistance, to report any bug or for suggestions
# please contact: sara.bosticardo@univr.it 
# 
#----------------------------------------------------------------------------------------------------------------------------------------------
#
# This script is created to run in a folder organized as follow:
# Main_Analysis_folder/PZ_number/Diffusion
# where "_number" should be the relative number assigned to each subject and the folder "Diffusion" of each subject contains: preprocessed data
#
#----------------------------------------------------------------------------------------------------------------------------------------------

import numpy as np
import amico
import commit
import os
import matplotlib.pyplot as plt
import nibabel as nib

spath = os.path.join('/...', '...', '...', '...', '...') #path pre-processed data
subj_list = [f for f in os.listdir(spath) if not f.startswith('.')]
subj_list.sort()

if len(subj_list) == 0:
    print(f"Warning! Files not found in {spath}. Please provide the data in the folder.")

for subj in subj_list:
    print(f"======================= SUBJ-{subj} =======================")

    Subj_data_path = os.path.join(spath, subj, 'Diffusion')
    path_smt = os.path.join(Subj_data_path, 'Tensor_fitting', 'SMT', 'SMT_upsampling', 'SMT_results_intra_upsampling.nii.gz')

    commit_subj = os.path.join(Subj_data_path, 'COMMIT_INVF')
    if not os.path.exists(commit_subj):
        os.makedirs(commit_subj)
    os.chdir(commit_subj)
    
    Tractography = os.path.join(Subj_data_path, 'Tractography_upsampling')
    FOD_folder = os.path.join(Tractography, 'FOD')
    T1_on_DTI = os.path.join(Tractography, 'T1_on_DTI')
    WM_FOD_norm = os.path.join(FOD_folder, 'WM_FODs_norm.nii.gz')
    tractogram = os.path.join(Subj_data_path, 'COMMIT_SZB', 'COMMIT', 'Results_StickZeppelinBall_COMMIT2', 'mitX_filtered_2.tck')
    T1_wm_mask = os.path.join(T1_on_DTI, 'WM_mask_to_b0.nii.gz')
    dwi_preproc_B1corr = os.path.join(Subj_data_path, 'dwi_preproc_upsaml_bet.nii.gz')
    TT_T1_brain_FS = os.path.join(Tractography, '5TT', "T1_brain_FS.nii.gz")
    whitematter_fsl = os.path.join(Tractography, '5TT', 'T1_wm.nii.gz')

    #################################################################################################
    #COMMIT MIELINA 
    #################################################################################################

    # Setting the input data
    commit.core.setup(ndirs=1)
    from commit import trk2dictionary
    trk2dictionary.run(
        ndirs = 1,
        filename_tractogram = tractogram,
        TCK_ref_image  = T1_wm_mask,
        filename_mask  = T1_wm_mask,
        path_out       = 'COMMIT',
        fiber_shift    = 0.5,
        min_seg_len    = 1e-3
    )

    # Setting parameters 
    mit = commit.Evaluation( '.', '.' )
    mit.set_config('doNormalizeSignal', False)
    mit.set_config('doMergeB0', False)
    mit.set_config('doNormalizeKernels', True)

    scheme = os.path.join(spath, 'scheme.scheme')    
    mit.load_data( path_smt, scheme )

    # Set model and generate the kernel
    mit.set_model( 'VolumeFractions' )
    mit.generate_kernels( regenerate=True )
    mit.load_kernels()

    # Load dictionary and buid the operator
    mit.load_dictionary( 'COMMIT' )
    
    mit.set_threads(1)
    mit.build_operator()
    
    # fitting
    mit.fit( tol_fun=1e-3, max_iter=1000 )
    mit.save_results() 

    # extracting non-zero streamlines

    Results = os.path.join(commit_subj, 'COMMIT', 'Results_VolumeFractions')

    streamline_weights = os.path.join(Results, 'streamline_weights.txt')
    mitX_filtered_txt = os.path.join(Results, 'mitX_filtered.txt')
    mitX_filtered_tck = os.path.join(Results, 'mitX_filtered.tck')

    cmd = f"tckedit -minweight 0.000000000001 -tck_weights_in {streamline_weights} -tck_weights_out {mitX_filtered_txt} {tractogram} {mitX_filtered_tck}"
    os.system(cmd)

    COMMIT_connectomes = os.path.join(Results, 'COMMIT_INVF_connectomes')
    if not os.path.exists(COMMIT_connectomes):
        os.makedirs(COMMIT_connectomes) 

    nodes = os.path.join(Subj_data_path, 'Connectome_upsampl', 'nodes_fixSGM_tob0.nii.gz')
    mitX_filtered_assignments = os.path.join(COMMIT_connectomes, 'mitX_filtered_assignments.txt')
    mitX_filtered_connectome = os.path.join(COMMIT_connectomes,'mitX_filtered_connectome.csv')

    cmd = f"tck2connectome -symmetric {mitX_filtered_tck} -out_assignments {mitX_filtered_assignments} {nodes} {mitX_filtered_connectome} -assignment_radial_search 2" 
    os.system(cmd) 

    #Creating the connectome sum(x)
    mitX_filtered_connectome_SumX = os.path.join(COMMIT_connectomes, f'mitX_filtered_connectome_SumX_INVF.csv')
    cmd = f"tck2connectome {mitX_filtered_tck} {nodes} {mitX_filtered_connectome_SumX} -tck_weights_in {mitX_filtered_txt} -stat_edge sum -symmetric -force"
    os.system(cmd)

    # computing the connectome mean(lenghts)
    mitX_filtered_connectome_meanL = os.path.join(COMMIT_connectomes, 'mitX_filtered_connectome_meanL.csv')
    cmd = f"tck2connectome -symmetric {mitX_filtered_tck} {nodes} {mitX_filtered_connectome_meanL} -scale_length -stat_edge mean -force"
    os.system(cmd)

    # computing the connectome sum(XbyL)
    connectome_sum_XbyL = os.path.join(COMMIT_connectomes, 'mitX_filtered_connectome_sumXbyL.csv') 
    cmd = f"tck2connectome -symmetric {mitX_filtered_tck} {nodes} {connectome_sum_XbyL} -scale_length -tck_weights_in {mitX_filtered_txt} -stat_edge sum -force"
    os.system(cmd)

    # computing the connectome sum(XbyL)/mean(L)
    connectome_sum_XbyL_gen = np.loadtxt(connectome_sum_XbyL, delimiter = ',')
    connectome_meanL_gen = np.loadtxt(mitX_filtered_connectome_meanL, delimiter = ',')
    connectome_sumXbyL_over_meanL = connectome_sum_XbyL_gen/connectome_meanL_gen
    connectome_sumXbyL_over_meanL = np.nan_to_num(connectome_sumXbyL_over_meanL)
    connectome_sumXbyL_over_meanL_save = os.path.join(COMMIT_connectomes, 'mitX_filtered_connectome_sumXbyL_over_meanL_INVF.csv')
    np.savetxt(connectome_sumXbyL_over_meanL_save, connectome_sumXbyL_over_meanL, fmt='%.16f', delimiter=',')

    kernels = os.path.join(commit_subj, 'kernels')

    print(f'rimuovo: {kernels}')
    cmd = f"rm -r {kernels}"
    os.system(cmd)

    print(f'rimuovo: {mitX_filtered_tck}')
    cmd = f"rm -r {mitX_filtered_tck}"
    os.system(cmd)

    commit_dict_folder = os.path.join(commit_subj, 'COMMIT')
    dicts = [d for d in os.listdir(commit_dict_folder) if 'dictionary' in d]
    for dict in dicts:
        print(f'Removing {os.path.join(commit_dict_folder, dict)}')
        os.remove(os.path.join(commit_dict_folder, dict))
